@@ -280,7 +280,7 @@ def node_decorate_dict(filename, target, source, toplevel):
         provenance.members[key] = member
 
         target_value = target.get(key)
-        if isinstance(value, collections.Mapping):
+        if isinstance(value, collections.abc.Mapping):
             node_decorate_dict(filename, target_value, value, toplevel)
         elif isinstance(value, list):
             member.elements = node_decorate_list(filename, target_value, value, toplevel)
@@ -295,7 +295,7 @@ def node_decorate_list(filename, target, source, toplevel):
         target_item = target[idx]
         element = ElementProvenance(filename, source, idx, toplevel)
 
-        if isinstance(item, collections.Mapping):
+        if isinstance(item, collections.abc.Mapping):
             node_decorate_dict(filename, target_item, item, toplevel)
         elif isinstance(item, list):
             element.elements = node_decorate_list(filename, target_item, item, toplevel)
@@ -569,7 +569,7 @@ def is_ruamel_str(value):
 #
 def is_composite_list(node):
 
-    if isinstance(node, collections.Mapping):
+    if isinstance(node, collections.abc.Mapping):
         has_directives = False
         has_keys = False
 
@@ -838,7 +838,7 @@ def composite_dict(target, source, path=None):
 
         target_value = target.get(key)
 
-        if isinstance(source_value, collections.Mapping):
+        if isinstance(source_value, collections.abc.Mapping):
 
             # Handle creating new dicts on target side
             if target_value is None:
@@ -853,7 +853,7 @@ def composite_dict(target, source, path=None):
                 # Add a new provenance member element to the containing dict
                 target_provenance.members[key] = source_provenance.members[key]
 
-            if not isinstance(target_value, collections.Mapping):
+            if not isinstance(target_value, collections.abc.Mapping):
                 raise CompositeTypeError(thispath, type(target_value), type(source_value))
 
             # Recurse into matching dictionary
@@ -914,7 +914,7 @@ RoundTripRepresenter.add_representer(SanitizedDict,
 #
 def node_sanitize(node):
 
-    if isinstance(node, collections.Mapping):
+    if isinstance(node, collections.abc.Mapping):
 
         result = SanitizedDict()
 
@@ -1052,7 +1052,7 @@ class ChainMap(collections.ChainMap):
 def node_chain_copy(source):
     copy = ChainMap({}, source)
     for key, value in source.items():
-        if isinstance(value, collections.Mapping):
+        if isinstance(value, collections.abc.Mapping):
             copy[key] = node_chain_copy(value)
         elif isinstance(value, list):
             copy[key] = list_chain_copy(value)
@@ -1065,7 +1065,7 @@ def node_chain_copy(source):
 def list_chain_copy(source):
     copy = []
     for item in source:
-        if isinstance(item, collections.Mapping):
+        if isinstance(item, collections.abc.Mapping):
             copy.append(node_chain_copy(item))
         elif isinstance(item, list):
             copy.append(list_chain_copy(item))
@@ -1080,7 +1080,7 @@ def list_chain_copy(source):
 def node_copy(source):
     copy = {}
     for key, value in source.items():
-        if isinstance(value, collections.Mapping):
+        if isinstance(value, collections.abc.Mapping):
             copy[key] = node_copy(value)
         elif isinstance(value, list):
             copy[key] = list_copy(value)
@@ -1097,7 +1097,7 @@ def node_copy(source):
 def list_copy(source):
     copy = []
     for item in source:
-        if isinstance(item, collections.Mapping):
+        if isinstance(item, collections.abc.Mapping):
             copy.append(node_copy(item))
         elif isinstance(item, list):
             copy.append(list_copy(item))
@@ -1132,7 +1132,7 @@ def node_final_assertions(node):
             raise LoadError(LoadErrorReason.TRAILING_LIST_DIRECTIVE,
                             "{}: Attempt to override non-existing list".format(provenance))
 
-        if isinstance(value, collections.Mapping):
+        if isinstance(value, collections.abc.Mapping):
             node_final_assertions(value)
         elif isinstance(value, list):
             list_final_assertions(value)
@@ -1140,7 +1140,7 @@ def node_final_assertions(node):
 
 def list_final_assertions(values):
     for value in values:
-        if isinstance(value, collections.Mapping):
+        if isinstance(value, collections.abc.Mapping):
             node_final_assertions(value)
         elif isinstance(value, list):
             list_final_assertions(value)
